const RDF_NS = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
const PFS_NS = "http://www.mozilla.org/2004/pfs-rdf#";

function nsRDFItemUpdater(aClientOS, aChromeLocale){
  this._rdfService = Components.classes["@mozilla.org/rdf/rdf-service;1"]
                        .getService(Components.interfaces.nsIRDFService);
  this._os = Components.classes["@mozilla.org/observer-service;1"]
                       .getService(Components.interfaces.nsIObserverService);

  var prefBranch = Components.classes["@mozilla.org/preferences-service;1"]
                             .getService(Components.interfaces.nsIPrefBranch);

  var app = Components.classes["@mozilla.org/xre/app-info;1"]
                      .getService(Components.interfaces.nsIXULAppInfo);
  this.appID = app.ID;
  this.buildID = app.platformBuildID;

  this.clientOS = aClientOS;
  this.chromeLocale = aChromeLocale;

  var prefBranch = Components.classes["@mozilla.org/preferences-service;1"]
                             .getService(Components.interfaces.nsIPrefBranch);
  this.dsURI = prefBranch.getCharPref("pfs.datasource.url");

}

nsRDFItemUpdater.prototype = {
  checkForPlugin: function (aPluginRequestItem){
    var dsURI = this.dsURI;
    dsURI = dsURI.replace(/%PLUGIN_MIMETYPE%/g, aPluginRequestItem.mimetype);
    dsURI = dsURI.replace(/%APP_ID%/g, this.appID);
    dsURI = dsURI.replace(/%APP_VERSION%/g, this.buildID);
    dsURI = dsURI.replace(/%CLIENT_OS%/g, this.clientOS);
    dsURI = dsURI.replace(/%CHROME_LOCALE%/g, this.chromeLocale);

	var pluginInfo = null;
    var mPrefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
    try 
    {
        var pref = "plugins.data.timeout." + aPluginRequestItem.mimetype;
        var timeout = Date.parse(mPrefs.getCharPref(pref));
        pluginInfo = new Object();
        var plugPref = "plugins.data." + aPluginRequestItem.mimetype + ".";
        pluginInfo.name = mPrefs.getCharPref(plugPref + "name");
        pluginInfo.pid = mPrefs.getCharPref(plugPref + "pid");
        pluginInfo.XPILocation = mPrefs.getCharPref(plugPref + "XPILocation");
        
        pluginInfo.version = mPrefs.getCharPref(plugPref + "version");
        pluginInfo.IconUrl = mPrefs.getCharPref(plugPref + "IconUrl");
        
        pluginInfo.InstallerShowsUI = mPrefs.getBoolPref(plugPref + "InstallerShowsUI");

        pluginInfo.manualInstallationURL = 
            mPrefs.getCharPref(plugPref + "manualInstallationURL");
        
        pluginInfo.requestedMimetype = mPrefs.getCharPref(plugPref + "requestedMimetype");
        pluginInfo.licenseURL = mPrefs.getCharPref(plugPref + "licenseURL");
        pluginInfo.needsRestart = mPrefs.getBoolPref(plugPref + "needsRestart");
        
        var date = new Date();
        if( date.getTime() > timeout)
        {
            pluginInfo = null;		
        }
    }
    catch (e)
    {
        pluginInfo = null;
    }

    if(pluginInfo != null)
    {
        gPluginInstaller.pluginInfoReceived(pluginInfo);
        return;
    }
    
    var timeoutstr = null;
    try
    {
        timeoutstr = mPrefs.getCharPref("plugins.data.timeout");
    }catch(e)
    {
        timeoutstr = null;
    }
    if(timeoutstr != null)
    {
        var glbtimeout = Date.parse(timeoutstr);
        var now = new Date();
        if( now.getTime() < glbtimeout)
        {
            return;
        }
    }
    
    var ds = this._rdfService.GetDataSource(dsURI);
    var rds = ds.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource)

    if (rds.loaded)
      this.onDatasourceLoaded(ds, aPluginRequestItem);
    else {
      var sink = ds.QueryInterface(Components.interfaces.nsIRDFXMLSink);
      sink.addXMLSinkObserver(new nsPluginXMLRDFDSObserver(this, aPluginRequestItem));
    }
  },

onDatasourceLoaded: function pfs_onDatasourceLoaded (aDatasource, aPluginRequestItem){
    var pluginInfo = null;
    var mPrefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
    // check the overriding pref
    var container = Components.classes["@mozilla.org/rdf/container;1"].
                  createInstance(Components.interfaces.nsIRDFContainer);
    var resultRes = this._rdfService.GetResource("urn:mozilla:plugin-results:" + aPluginRequestItem.mimetype);
    var pluginList = aDatasource.GetTarget(resultRes, this._rdfService.GetResource(PFS_NS+"plugins"), true);

    container = Components.classes["@mozilla.org/rdf/container;1"].createInstance(Components.interfaces.nsIRDFContainer);
   try {
      container.Init(aDatasource, pluginList);

      var children = container.GetElements();
      var target;

      // get the first item
      var child = children.getNext();
      if (child instanceof Components.interfaces.nsIRDFResource){
        var name = this._rdfService.GetResource("http://www.mozilla.org/2004/pfs-rdf#updates");
        target = aDatasource.GetTarget(child, name, true);
      }

        container.Init(aDatasource, target);
        target = null;
        children = container.GetElements();

        var child = children.getNext();
        if (child instanceof Components.interfaces.nsIRDFResource){
          target = child;
        }

        function getPFSValueFromRDF(aValue, aDatasource, aRDFService){
          var rv = null;

          var myTarget = aDatasource.GetTarget(target, aRDFService.GetResource(PFS_NS + aValue), true);
          if (myTarget)
            rv = myTarget.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;

          return rv;
        }    
        var count =  getPFSValueFromRDF("count", aDatasource, this._rdfService);

        var glbtimeout =  getPFSValueFromRDF("timeout", aDatasource, this._rdfService);
        if(glbtimeout != null)
        {
            var futdate = new Date();
            var expdate = futdate.getTime();
            expdate += glbtimeout*1000;
            futdate.setTime(expdate);
            mPrefs.setCharPref("plugins.data.timeout",futdate.toGMTString());   
        }

        for(var i = count; i > 0; i--)
        {
            pluginInfo = new Object();
            pluginInfo.name = getPFSValueFromRDF("name_"+i, aDatasource, this._rdfService);
						if(pluginInfo.name == null && i==1)
						{
								pluginInfo.pid = -1;
								break;
						}
            pluginInfo.pid = getPFSValueFromRDF("guid_"+i, aDatasource, this._rdfService);
            pluginInfo.version = getPFSValueFromRDF("version_"+i, aDatasource, this._rdfService);
            pluginInfo.XPILocation = getPFSValueFromRDF("XPILocation_"+i, aDatasource, this._rdfService);
            try
            {
                pluginInfo.IconUrl = getPFSValueFromRDF("IconUrl_"+i, aDatasource, this._rdfService);
                pluginInfo.InstallerShowsUI = getPFSValueFromRDF("InstallerShowsUI_"+i, aDatasource, this._rdfService);
                pluginInfo.manualInstallationURL = getPFSValueFromRDF("manualInstallationURL_"+i, aDatasource, this._rdfService);
                pluginInfo.requestedMimetype = getPFSValueFromRDF("requestedMimetype_"+i, aDatasource, this._rdfService);
                pluginInfo.licenseURL = getPFSValueFromRDF("licenseURL_"+i, aDatasource, this._rdfService);
                pluginInfo.needsRestart = getPFSValueFromRDF("needsRestart_"+i, aDatasource, this._rdfService);
                pluginInfo.timeout = getPFSValueFromRDF("timeout_"+i, aDatasource, this._rdfService);

            }
            catch (ex){}
            if( pluginInfo.InstallerShowsUI == null) pluginInfo.InstallerShowsUI=false;
            if( pluginInfo.needsRestart == null) pluginInfo.needsRestart=false;
            var futdate = new Date();
            var expdate = futdate.getTime();
            expdate += pluginInfo.timeout*1000;
            futdate.setTime(expdate);
						if(pluginInfo.pid == -1)
						{
                pluginInfo.IconUrl = "";
                pluginInfo.InstallerShowsUI = "";
                pluginInfo.manualInstallationURL = "";
                pluginInfo.requestedMimetype = "";
                pluginInfo.licenseURL = "";
                pluginInfo.needsRestart = "";
                pluginInfo.timeout = "";
						}
						else
						{
            	var plugPref = "plugins.data." + pluginInfo.requestedMimetype + ".";
            	mPrefs.setCharPref(plugPref + "name", pluginInfo.name );
            	mPrefs.setCharPref(plugPref + "pid", pluginInfo.pid );
            	mPrefs.setCharPref(plugPref + "version", pluginInfo.version  );
            	mPrefs.setCharPref(plugPref + "IconUrl",  pluginInfo.IconUrl);
            	mPrefs.setCharPref(plugPref + "XPILocation", pluginInfo.XPILocation);
            	mPrefs.setBoolPref(plugPref + "InstallerShowsUI", pluginInfo.InstallerShowsUI);
            	mPrefs.setCharPref(plugPref + "manualInstallationURL", pluginInfo.manualInstallationURL);
            	mPrefs.setCharPref(plugPref + "requestedMimetype", pluginInfo.requestedMimetype);
            	mPrefs.setCharPref(plugPref + "licenseURL", pluginInfo.licenseURL );
            	mPrefs.setBoolPref(plugPref + "needsRestart", pluginInfo.needsRestart);
            	mPrefs.setCharPref(plugPref + "timeout", futdate.toGMTString());
        	}
   		}
   }
   catch (ex){}

		
    gPluginInstaller.pluginInfoReceived(pluginInfo);
  },

  onDatasourceError: function pfs_onDatasourceError (aPluginRequestItem, aError){
    this._os.notifyObservers(aPluginRequestItem, "error", aError);
    gPluginInstaller.pluginInfoReceived(null);
  },
};

function nsPluginXMLRDFDSObserver(aUpdater, aPluginRequestItem){
  this._updater = aUpdater;
  this._item    = aPluginRequestItem;
}

nsPluginXMLRDFDSObserver.prototype = 
{ 
  _updater  : null,
  _item     : null,

  // nsIRDFXMLSinkObserver
  onBeginLoad: function(aSink){},
  onInterrupt: function(aSink){},
  onResume: function(aSink){},
  onEndLoad: function(aSink){
    aSink.removeXMLSinkObserver(this);
    
    var ds = aSink.QueryInterface(Components.interfaces.nsIRDFDataSource);
    this._updater.onDatasourceLoaded(ds, this._item);
  },
  
  onError: function(aSink, aStatus, aErrorMsg){  
    aSink.removeXMLSinkObserver(this);   
    this._updater.onDatasourceError(this._item, aStatus.toString());
  }
};
